﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;
using TestingLinqToSQLMockA;
using Rhino.Mocks;

namespace TestingLinqToSQL.Tests
{
    [TestFixture]
    public class ProductControllerTests
    {
        MockRepository m_Mocks;
        IProductRepository m_ProductRepos;
        Category m_MockedCategory;
        Product m_MockedProduct;

        [SetUp]
        public void Setup()
        {
            m_Mocks = new MockRepository();
            m_ProductRepos = m_Mocks.CreateMock<IProductRepository>();
            ProductController.ProductRepository = m_ProductRepos;

            m_MockedCategory = MockCategory();
            m_MockedProduct = MockProduct();
        }

        [TearDown]
        public void Teardown()
        {
            m_Mocks = null;
        }

        private Category MockCategory()
        {
            Category c = new Category();
            c.id = 1;
            c.Title = "Microsoft Software";
            c.Description = "All the latest Microsoft releases.";
            return c;

        }

        private Product MockProduct()
        {
            Product p = new Product();
            p.id = 1;
            p.Title = "Visual Studio 2008";
            p.Description = "IDE";
            p.Price = 1000;
            return p;
        }

        [Test]
        public void GetCategories_NoContent_ReturnEmptyList()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetCategories()).Return(new List<Category>());
            }

            using (m_Mocks.Playback())
            {
                List<Category> categories = ProductController.GetCategories();
                Assert.IsNotNull(categories);
                Assert.AreEqual(0, categories.Count);
            }
        }

        [Test]
        public void GetCategories_SimpleContent_ReturnPopulatedList()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetCategories()).Return(new List<Category> { m_MockedCategory });
            }

            using (m_Mocks.Playback())
            {
                List<Category> categories = ProductController.GetCategories();
                Assert.IsNotNull(categories);
                Assert.AreEqual(1, categories.Count);
                Assert.AreEqual("Microsoft Software", categories[0].Title);
            }
        }

        [Test]
        public void GetCategoryByName_NameOfValidCategory_ReturnCategoryObject()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetCategory("")).IgnoreArguments().Return(m_MockedCategory);
            }

            using (m_Mocks.Playback())
            {
                Category c = ProductController.GetCategory("Microsoft Software");

                Assert.AreEqual("Microsoft Software", c.Title);
                Assert.AreEqual("All the latest Microsoft releases.", c.Description);
            }
        }

        [Test]
        public void GetProductsForCategory_ValidCategory_EmptyList()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetProductsForCategory(m_MockedCategory)).IgnoreArguments().Return(new List<Product>());
            }

            using (m_Mocks.Playback())
            {
                List<Product> products = ProductController.GetProductsForCategory(m_MockedCategory);
                Assert.AreEqual(0, products.Count);
            }
        }

        [Test]
        public void GetProductsForCateogry_ValidCategoryWithProduct_PopulatedList()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetProductsForCategory(m_MockedCategory)).Return(new List<Product> { m_MockedProduct });
            }

            using (m_Mocks.Playback())
            {
                List<Product> products = ProductController.GetProductsForCategory(m_MockedCategory);
                Assert.AreEqual(1, products.Count);
                Assert.AreEqual("Visual Studio 2008", products[0].Title);
            }
        }

        [Test]
        public void GetProductByID_ValidProductID_ReturnProductID()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetProduct(1)).Return(m_MockedProduct);
            }

            using (m_Mocks.Playback())
            {
                Product p = ProductController.GetProduct(1);
                Assert.AreEqual(p.Title, m_MockedProduct.Title);
            }
        }
    }
}
